#coding: utf-8
import binascii
import json
import random
import socket
import threading
import time
import traceback


class SendObd():
    def __init__(self):
        self.host = "10.100.12.32"
        self.port = 9008
        self.timeOut = 2
        self.BUF_SIZE = 1024
        self.carId = 202006115001                  #设置开始发送的车机号
        self.totalCar = 5000                         #设置将会累加的车机号

        self.carIdArr = ["202003060165","202003060166"]

    #获取要发送的OBD数据（M500车机）
    def getOBDMsg_M500(self,carId,mileage):
        timeStamp = time.time()
        timeArray = time.localtime(int(timeStamp - 8 * 3600))
        dateTimeM = time.strftime("%Y-%m-%d %H:%M:%S", timeArray)
        timeHex = self.getUTCTime(dateTimeM)
        totalMileage = self.int2hexStringByBytes(mileage,4)
        # OBD_msg = "4040007000064d" + str(carId) + "001201" + timeHex + "26d7fffff0000000000505000000143c00000bb80100000fa00000000a0000000000005e60723b723b39331e100055320000001312001007d0001e0000000000000096000000280096ffff3e0001f40000003e00000000000000000000007213"
        OBD_msg = "4040007000064d" + str(carId) + "001201" + timeHex + "ffffffffff000000000505000000143c00000bb801" + totalMileage + "0000000a0000000000005e60723b723b39331e100055320000001312001007d0001e0000000000000096000000280096ffff3e0001f40000003e00000000000000000000007213"

        OBD_msg = OBD_msg[:-4] + self.crc16(OBD_msg[:-4])
        msg = OBD_msg
        return msg

    ############################################
    # 发送一条消息
    ############################################
    def sendMsg(self,msg):
        msg = msg
        client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        client.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)  # 在客户端开启心跳
        client.settimeout(self.timeOut)
        startTime = int(time.time() * 1000)
        try:
            client.connect((self.host, self.port))
            client.send(binascii.a2b_hex(msg))
            print("发送消息：" + msg)
        except BaseException as e:
            client.close()
            print("连接超时，socket断开")
            return
        try:
            data = client.recv(self.BUF_SIZE)
            print("接收消息：" + str(data))
        except BaseException as e:
            traceback.print_exc()
            client.close()
            print('socket 接收消息超时！')
            return
        endTime = int(time.time() * 1000)
        timeExpend = endTime - startTime
        client.close()

    ####################################################
    #       定义生成校验字段的函数（M500 校验方式）
    #       inputStr:需要传入一个已经转换为16进制的字符串
    #####################################################
    # add crc 16 check at the end of the string
    def crc16(self,inputStr):
        inputStrByte = bytes.fromhex(inputStr)
        crc = 0xFFFF
        for i in range(0, len(inputStrByte)):
            for j in range(0, 8):
                c15 = (crc >> 15) == 1
                bit = ((inputStrByte[i] >> (7 - j)) & 1) == 1
                crc <<= 1
                crc &= 0xFFFF
                if c15 ^ bit:
                    crc ^= 0x1021
        crc = str(hex(crc))
        crc = self.leftPad(crc[2:], 4)
        # outputStr = inputStr + crc
        outputStr = crc
        return outputStr
    # pad zero to the left of the string if not long enough
    def leftPad(self,inputStr, strLen):
        if (strLen > len(inputStr)):
            outputStr = "0000000000000000000000000000000000000000" + inputStr
            outputStr = outputStr[len(outputStr) - strLen:]
            return outputStr
        else:
            return inputStr
    # pad zero to the right of the string if not long enough
    def rightPad(self,inputStr, strLen):
        if (strLen > len(inputStr)):
            outputStr = inputStr + "0000000000000000000000000000000000000000"
            outputStr = outputStr[: strLen]
            return outputStr
        else:
            return inputStr

    #####################################################
    #               将UTC时间转换为16进制，
    #        例如：2020-01-02   20:30:00 （年取后面2字节）则将20,01，02,20,30,00 转换为对应的6个字节
    #        theTime:传入一个类似：2020-01-03 13:05:13的一个字符串
    #####################################################
    def getUTCTime(self,theTime):
        # 获取当前时间，时间格式为：2020-01-03 13:05:13
        # now_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        # 将2020-01-03 13:05:13时间格式转换为一个数组
        # timeStr = "2020-01-03 13:05:13"
        timeStr = theTime
        timeArr = []
        timeArr.append(timeStr[2:4])
        timeArr.append(timeStr[5:7])
        timeArr.append(timeStr[8:11])
        timeArr.append(timeStr[11:13])
        timeArr.append(timeStr[14:16])
        timeArr.append(timeStr[17:19])
        UTCTime = ""
        for i in range(0, len(timeArr)):
            UTCTime += self.int2hexString(int(timeArr[i]))
        return UTCTime

    #####################################################
    #               数字转换为16进制字符串
    #####################################################
    def int2hexString(self, num):
        hexStr = hex(num)[2:]
        if (len(hexStr) % 2) == 1:
            hexStr = "0" + hexStr
        return hexStr

    #####################################################
    #               数字转换为16进制字符串，通过传入字节数可自动补0
    #               传入数据格式所占字节数
    #####################################################
    def int2hexStringByBytes(self, num,bytescount=1):
        hexStr = hex(num)[2:]
        while len(hexStr) < (bytescount * 2):
            hexStr = "0" + hexStr
        return hexStr

    #####################################################
    #               发送两条总里程不一样的OBD数据
    #####################################################
    def sendTwoObdMsg(self,carId,mileage):
        theMileage = mileage
        msg = self.getOBDMsg_M500(carId,theMileage)
        self.sendMsg(msg)
        time.sleep(5)
        theMileage = mileage + random.randint(2000,10000)
        msg = self.getOBDMsg_M500(carId, theMileage)
        self.sendMsg(msg)


    #####################################################
    #               使用多个车机号批量发送
    #####################################################
    def sendMultMsgService(self):
        for i in range(0,self.totalCar):
            carId = str(self.carId + i)
            print(carId)
            theThread = threading.Thread(target=self.sendTwoObdMsg, args=(carId,220000))
            theThread.start()
            time.sleep(0.01)
            print(theThread.getName())

    #####################################################
    #               使用多个车机号批量发送(数组方式)
    #####################################################
    def sendMultMsgServiceArr(self):
        for i in range(0,len(self.carIdArr)):
            carId = str(self.carIdArr[i])
            print(carId)
            theThread = threading.Thread(target=self.sendTwoObdMsg, args=(carId,10000))
            theThread.start()
            # time.sleep(0.1)
            print(theThread.getName())

if __name__ == "__main__":
    obj = SendObd()
    obj.sendMultMsgService()
    # obj.sendMultMsgServiceArr()